/*
 * Connectivity Model
 * Communications Electronic Warfare Branch
 * IEPD/SLAD
 * US Army Research Laboratory
 *
 * (See author section below for contact info.)
 */
package filter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StreamTokenizer;

/**
 *
 * @author Mike Markowski (mm@arl.army.mil)
 * AMSRD-ARL-SL-EW
 * APG, MD  21005
 */
public class Config {

    private BufferedReader in = null;
    private PrintWriter out = null;
    private File configFile = null;
    private FilterModel[][] dspFilters = null;
    private FilterModel[][] crystalFilters = null;
    private StreamTokenizer st;
    private String homeDir = System.getProperty("user.home");
    private String configDir = homeDir + File.separator + ".k2ab3ap";
    private String filterConfig = configDir + File.separator + "filters";

    public Config() {
    }

    public void setCrystalFilters(FilterModel[][] crystalFilters) {
        this.crystalFilters = crystalFilters;
    }

    public void setDspFilters(FilterModel[][] dspFilters) {
        this.dspFilters = dspFilters;
    }

    public BufferedReader openConfigFileForReading() {

        BufferedReader br = null;
        configFile = new File(homeDir);
        if (!configFile.canRead()) {
            return null;
        }
        configFile = new File(configDir);
        if (!configFile.exists()) {
            configFile.mkdir();
            return null;
        } else if (!configFile.isDirectory()) {
            return null;
        }
        configFile = new File(filterConfig);
        if (configFile.exists() && !configFile.canRead()) {
            return null;
        }
        try {
            br = new BufferedReader(new FileReader(configFile));
        } catch (Exception e) {
            System.out.println("Can't open file " + configFile.getAbsolutePath() +
                    " for reading.");
            return null;
        }
        return br;
    }

    public PrintWriter openConfigFileForWriting() {

        PrintWriter bw = null;
        configFile = new File(homeDir);
        if (!configFile.canRead()) {
            return null;
        }
        configFile = new File(configDir);
        if (!configFile.exists()) {
            configFile.mkdir();
            return null;
        } else if (!configFile.isDirectory()) {
            return null;
        }
        configFile = new File(filterConfig);
        if (configFile.exists() && !configFile.canRead()) {
            return null;
        }
        try {
            bw = new PrintWriter(
                    new BufferedWriter(new FileWriter(configFile)));
        } catch (Exception e) {
            System.out.println("Can't open file " + configFile.getAbsolutePath() +
                    " for writing.");
            return null;
        }
        return bw;
    }

    public void readConfigFile() {

        int f, m;
        String name;
        if (in != null) {
            in = openConfigFileForReading();
        }
        st = new StreamTokenizer(in);
        st.resetSyntax();
        st.whitespaceChars(0, 32);
        st.whitespaceChars(127, 127);
        st.wordChars(33, 126);
        st.slashSlashComments(true);
        st.slashStarComments(true);
        st.eolIsSignificant(false);
        while ((name = nextToken()) != null) {
            if (name.equals("XFIL")) {
                nextToken(); // "BW"
                nextToken(); // mode array index.
                m = (int) st.nval;
                nextToken(); // filter array index.
                f = (int) st.nval;
                nextToken(); // XFIL bandwidth.
                crystalFilters[m][f].setBandwidth(st.nval);
            } else if (name.equals("AFIL")) {
                String s = nextToken();
                nextToken();
                if (s.equals("BW")) {
                    m = (int) st.nval;
                    nextToken();
                    f = (int) st.nval;
                    nextToken();
                    dspFilters[m][f].setBandwidth(st.nval);
                } else if (s.equals("BYPASS")) {
                    dspFilters[0][0].setDspBypass(true);
                } else if (s.equals("CTR")) {
                    m = (int) st.nval;
                    nextToken();
                    f = (int) st.nval;
                    nextToken();
                    dspFilters[m][f].setCenterFreq(st.nval);
                } else if (s.equals("GAIN")) {
                    m = (int) st.nval;
                    nextToken();
                    for (int i = 0; i < 4; i++) {
                        dspFilters[m][i].setFilterGain(st.nval);
                    }
                } else if (s.equals("HICUT")) {
                    m = (int) st.nval;
                    nextToken();
                    f = (int) st.nval;
                    nextToken();
                    dspFilters[m][f].setHighFreqCutoff(st.nval);
                } else if (s.equals("LOCUT")) {
                    m = (int) st.nval;
                    nextToken();
                    f = (int) st.nval;
                    nextToken();
                    dspFilters[m][f].setLowFreqCutoff(st.nval);
                } else if (s.equals("NARROW")) {
                    for (int i = 0; i < 4; i++) {
                        dspFilters[0][i].setCwNarrow(true);
                    }
                } else if (s.equals("NR")) {
                    dspFilters[0][0].setNoiseReductionLevel((int) st.nval);
                } else if (s.equals("NRBETA")) {
                    dspFilters[0][0].setNrBeta((int) st.nval);
                } else if (s.equals("NRGAIN")) {
                    m = (int) st.nval;
                    nextToken();
                    for (int i = 0; i < 4; i++) {
                        dspFilters[m][i].setDenoiserGain(st.nval);
                    }
                } else if (s.equals("NRHOLD")) {
                    dspFilters[0][0].setNrHold((int) st.nval);
                } else if (s.equals("NTBETA")) {
                    dspFilters[0][0].setNtBeta((int) st.nval);
                } else if (s.equals("NTHOLD")) {
                    dspFilters[0][0].setNtHold((int) st.nval);
                }
            }
        }
    }

    private String nextToken() {

        String name;

        name = null;
        try {
            st.nextToken();
        } catch (Exception e) {
            System.out.println("Can't get next token.");
        }
        switch (st.ttype) {
            case StreamTokenizer.TT_WORD:
                name = st.sval;
                break;
            case StreamTokenizer.TT_EOF:
                break;
            case StreamTokenizer.TT_EOL:
                break;
            case StreamTokenizer.TT_NUMBER:
//                name = Double.toString(st.nval);
                break;
            default:
                System.out.println("readString: this should never happen.");
                break;
        }
        //System.out.println("Token: " + name);
        return name;

    }

    public void writeConfigFile() {
        if (crystalFilters == null) {
            return;
        }
        try {
            in.close(); // Close configFile's BufferedReader.
        } catch (IOException ex) {
            System.out.println(getClass() +
                    ".writeConfigFile: couldn't close config file.");
            return;
        }
        if (configFile != null && configFile.exists()) {
            configFile.delete();
        }
        out = openConfigFileForWriting();
        if (dspFilters[0][0].isDspBypass()) {
            out.println("BYPASS");
        }
        for (int m = 0; m < crystalFilters.length; m++) {
            for (int f = 0; f < crystalFilters[0].length; m++) {
                FilterModel fm = dspFilters[m][f];
                out.println("XFIL BW " + m + f + fm.getBandwidth());
            }
        }
        if (dspFilters != null && dspFilters[0][0] != null) {
            // These settings hold for all KDSP2 filters.
            FilterModel fm = dspFilters[0][0];
            out.println("AFIL NR " + fm.getNoiseReductionLevel());
            out.println("AFIL NRBETA " + fm.getNrBeta()); // CW & SSB
            out.println("AFIL NRHOLD " + fm.getNrHold()); // CW & SSB
            out.println("AFIL NTBETA " + fm.getNtBeta()); // SSB
            out.println("AFIL NTHOLD " + fm.getNtHold()); // SSB
        }
        for (int m = 0; m < dspFilters.length; m++) {
            // Filters have per mode (CW, SBS, RTTY) gain.
            out.println("AFIL GAIN " + m + dspFilters[m][0].getFilterGain());
            if (m == 0 && dspFilters[m][0].isCwNarrow()) {
                out.println("AFIL NARROW");
            }
            if (m != 2) {
                // CW and SSB have a denoiser gain.
                out.println("AFIL NRGAIN " + m + dspFilters[m][0].
                        getDenoiserGain());
            }
            for (int f = 0; f < dspFilters[0].length; m++) {
                FilterModel fm = dspFilters[m][f];
                out.println("AFIL BW " + m + f + fm.getBandwidth());
                out.println("AFIL CTR " + m + f + fm.getCenterFreq());
                out.println("AFIL HICUT " + m + f + fm.getHighFreqCutoff());
                out.println("AFIL LOCUT " + m + f + fm.getLowFreqCutoff());
            }
        }
        out.close();
    }
}
