package antbw;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Observable;
import util.Serial;

public class Measurer extends Observable implements Runnable {

    private enum State {

        PAUSE, PLAY, STOP
    };
    private double freqStart;
    private double freqStop;
    private int numSamples;
    private int txPowerOrig, txPower;
    private File outputFile;
    private Serial rigPort, w1Port;
    private State state;

    public Measurer() {

        state = State.STOP;
    }

    public Serial getRigPort() {
        return rigPort;
    }

    public void setRigPort(Serial rigPort) {
        this.rigPort = rigPort;
    }

    public Serial getW1Port() {
        return w1Port;
    }

    public void setW1Port(Serial w1Port) {
        this.w1Port = w1Port;
    }

    @Override
    public void run() {
        txPowerOrig = getRigTxPower();
        rigTxPower(txPower);
        scan();
    }

    private void init() {
    }

    public void pause() {
        if (state == State.PAUSE) {
            play();
        } else {
            state = State.PAUSE;
        }
    }

    public void play() {
        if (state == State.PLAY) {
            return;
        }
        state = State.PLAY;
    }

    public void stop() {
        state = State.STOP;
    }

    /**
     * Read SWR from W1 watt meter.
     */
    private double w1Fwd() {
        byte[] response = new byte[8];
        w1Port.serialWrite("F", 60); // 60 msec timeout.
        w1Port.serialRead(response);
        if (response[0] != 'F') {
            String r = new String(response);
            System.out.println("SWR read error: '" + r + "'");
        }
        String numStr = new String(response, 1, 4);
        return Double.valueOf(numStr);
    }

    /**
     * Read SWR from W1 watt meter.
     */
    private double w1Rfl() {
        byte[] response = new byte[8];
        w1Port.serialWrite("R", 60); // 60 msec timeout.
        w1Port.serialRead(response);
        if (response[0] != 'R') {
            String r = new String(response);
            System.out.println("SWR read error: '" + r + "'");
        }
        String numStr = new String(response, 1, 4);
        return Double.valueOf(numStr);
    }

    /**
     * Read SWR from W1 watt meter.
     */
    private double w1Swr() {
        byte[] response = new byte[8];
        w1Port.serialWrite("S", 60); // 60 msec timeout.
        w1Port.serialRead(response);
        if (response[0] != 'S') {
            String r = new String(response);
            System.out.println("SWR read error: '" + r + "'");
        }
        String numStr = new String(response, 1, 4);
        return Double.valueOf(numStr);
    }

    private void cwMode() {
        byte[] response = new byte[8];
        rigPort.serialWrite("MD3;", 60); // 60 msec timeout.
        rigPort.serialRead(response); // Throw away response.
    }

    private void tune() {
        byte[] response = new byte[8];
        rigPort.serialWrite("SWH16;", 60); // 60 msec timeout.
        rigPort.serialRead(response); // Throw away response.
    }

    private int getRigTxPower() {
        byte[] response = new byte[8];
        String cmd = String.format("PC;");
        rigPort.serialWrite(cmd, 60); // 60 msec timeout.
        rigPort.serialRead(response); // Throw away response.
        String pwrStr = new String(response, 2, 3);
        return Integer.valueOf(pwrStr);
    }

    private void rigTxPower(int pwr_W) {
        byte[] response = new byte[8];
        String cmd = String.format("PC%03d;", pwr_W);
        rigPort.serialWrite(cmd, 60); // 60 msec timeout.
        rigPort.serialRead(response); // Throw away response.
    }

    private void tuneToFreq(int freq_Hz) {
        byte[] response = new byte[20];
        String cmd = String.format("FA%011d;", freq_Hz);
        rigPort.serialWrite(cmd, 100); // 60 msec timeout.
        rigPort.serialRead(response);
        response[15] = '\0';
        String numStr = new String(response, 0, 14);
//        if (!cmd.equals(numStr)) {
//            System.out.println("Freq set error!  " + cmd + " != " + numStr);
//        }
    }

    private void rigTxMode() {
        rigPort.serialWrite("TX;", 60); // 60 msec timeout.
    }

    private void rigRxMode() {
        rigPort.serialWrite("RX;", 60); // 60 msec timeout.
    }

    private void scan() {
        BufferedWriter out;
        try {
            out = new BufferedWriter(new FileWriter(outputFile));
        } catch (IOException ex) {
            System.out.println("Couldn't open " + outputFile.getAbsoluteFile()
                    + " for writing.");
            return;
        }

        double freqStep = (freqStop - freqStart) / (numSamples - 1);
        int n = 0;
        for (double f = freqStart; f <= freqStop; f += freqStep) {
            if (state == State.STOP) {
                rigRxMode();
                break;
            }
            if (state == State.PAUSE) {
                try {
                    Thread.sleep(500);
                } catch (InterruptedException ex) {
                }
            } else {
                double swr = measureSwr(f);
                try {
                    out.write(f + " " + swr + "\n");
                    // Write freq/swr values to file.
                } catch (IOException ex) {
                    System.out.println("Couldn't write to "
                            + outputFile.getAbsoluteFile());
                }
                n++;
                setChanged();
                notifyObservers(new Integer(100 * n / numSamples));
            }
        }
        try {
            out.close();
        } catch (IOException ex) {
            System.out.println("Couldn't close file "
                    + outputFile.getAbsoluteFile());
        }
        rigTxPower(txPowerOrig);
        setChanged();
        notifyObservers(new Integer(100)); // Done measuring.
    }

    private double measureSwr(double freq_kHz) {

        // 1. Tune to frequency.
        tuneToFreq((int) (freq_kHz * 1e3));
        tune(); // Tune on.

        // 2. Put transceiver in transmit mode.
//        rigTxMode();
        try {
            Thread.sleep(1000);
        } catch (InterruptedException ex) {
        }

        // 3. Retrieve watt metere SWR reading.
        double swr;
        if (true) {
            swr = w1Swr();
        } else {
            double fwd = w1Fwd();
            double rfl = w1Rfl();
            if (w1Fwd() == 0) {
                swr = 1;
            } else {
                double g = Math.sqrt(rfl / fwd);
                swr = (1 + g) / (1 - g);
            }
        }

        try {
            Thread.sleep(500);
        } catch (InterruptedException ex) {
        }

        tune(); // Tune off

        // 4. Put transceiver in receive mode.
//        rigRxMode();

        return swr;
    }

    public double getFreqStart() {
        return freqStart;
    }

    public void setFreqStart(double freqStart) {
        this.freqStart = freqStart;
    }

    public double getFreqStop() {
        return freqStop;
    }

    public void setFreqStop(double freqStop) {
        this.freqStop = freqStop;
    }

    public int getNumSamples() {
        return numSamples;
    }

    public void setNumSamples(int numSamples) {
        this.numSamples = numSamples;
    }

    public File getOutputFile() {
        return outputFile;
    }

    public void setOutputFile(File outputFile) {
        this.outputFile = outputFile;
    }

    public int getTxPower() {
        return txPower;
    }

    public void setTxPower(int txPower) {
        this.txPower = txPower;
    }
}
