package serialComms;

import util.Serial;

/**
 * This object deals both decoding the response from a K2 "DS" command and
 * for displaying it on-screen.  There is also a method, forceDisplay(), that
 * is useful to force this object to put something on the display other than
 * data from a DS command.
 *
 * From the KIO2 manual, the response to a "DS;" command is:
 *
 * GET format: DS; (no data). Returns everything needed to reproduce
 * the information shown on the K2 LCD. The format of K2's response
 * message is: DSttttttttaf; where <tttttttt> is the LCD text and
 * decimal point data, <a> is annunciator data, and <f> is annunciator
 * flash data.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br>
 * March 2007
 */
public class DSdecoder {

    /**
     * 7 (MSB) of each byte is used to indicate whether the decimal point to
     * the left of each character is on (1) or off (0). The remaining 7 bits
     * (b6-b0) contain an ASCII character that corresponds to the displayed
     * character.  Array index i holds decimal ponit info for bit i.
     */
    private boolean[] decimalPoints;
    byte[] ds;

    /** Create the decoder as well as set of 7 segment digit displays. */
    DSdecoder() {
        decimalPoints = new boolean[8];
    }

    /**
     * Return array of decimal point statuses.
     * 7 (MSB) of each byte is used to indicate whether the decimal point to
     * the left of each character is on (1) or off (0). The remaining 7 bits
     * (b6-b0) contain an ASCII character that corresponds to the displayed
     * character.  Array index i holds decimal ponit info for bit i.
     *
     * N.B. Must be called <i>after</i> getDisplay() is called.
     * @return array of decimal point statuses.
     */
    public boolean[] getDecimalPoints() {
        return decimalPoints;
    }

    private byte[] readK2Display(Serial sport) {

        int bytesRead = 0;
        int cycle = 0;
        ds = new byte[64];
        String s = "";
        do {
            sport.serialWrite("DS;", 300);
            bytesRead = sport.serialRead(ds);
            cycle++;
            if (cycle > 5) {
                System.out.println("Couldn't read display!");
                System.exit(1);
            }
            s = new String(ds);
        } while (bytesRead == 0 || s.indexOf(';') == -1);
        return ds;
    }

    public String getDisplay(Serial sport, boolean displayDecimalPoints) {

        ds = readK2Display(sport);
        return getOldDisplay(displayDecimalPoints);
    }

    public String getOldDisplay(boolean displayDecimalPoints) {

        if (ds == null) {
            return "";
        }

        String display;
        display = "";
        for (int i = 2; i < 10; i++) {
            boolean dp = ((ds[i] & 0x80) != 0);
            decimalPoints[i - 2] = dp;
            if (displayDecimalPoints && dp) {
                display += ".";
            }
            char b = (char) (ds[i] & 0x7f); // Clear MSBit.
            switch (b) {
                case '<':
                    display += 'L';
                    break;
                case '>':
                    display += '-';
                    break;
                case '@':
                    display += ' ';
                    break;
                case 'K':
                    display += 'H';
                    break;
                case 'M':
                    display += 'N';
                    break;
                case 'Q':
                    display += 'O';
                    break;
                case 'V':
                    display += 'U';
                    break;
                case 'W':
                    display += 'I';
                    break;
                case 'X':
                    display += 'C';
                    break;
                case 'Z':
                    display += 'c';
                    break;
                case '[':
                    display += 'r';
                    break;
                default:
                    display += b;
                    break;
            }
        }
        System.out.println("DS: '" + display.trim() + "'");
        return display.trim();
    }
}
