package k2;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import util.Serial;

/**
 * The K2 object does its best to control a real K2 and to mimic the on-screen
 * display of that K2 on its own virtual K2 display. At its essence, the program
 * waits for mouse clicks, sends the appropriate command to the K2, asks the K2
 * what its display looks like, and then copies the display itself. It wakes up
 * every 100 ms - faster would be better but might be too much for older
 * machines (or maybe not?). Maybe this should be an argument the user
 * specifies.
 *
 * <p>The K2, however, does not always share with others. For instance, during
 * direct frequency entry the K2 doesn't respond correctly, so the virtual K2
 * display is out of sync. Try it and see. Same goes for menu entries. Variable
 * and value are never underlined because the K2 doesn't send the information
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br> March 2007
 */
public class K2 {

    static final int TIMEOUT = 100; // milliseconds, timer setting.
    static final int HOLD_DELAY = 2; // tap < HOLD_DELAY*TIMEOUTS < hold

    public enum Mode {

        NORMAL, MENU, EDIT, DSP, POWER
    }
    DSdecoder dsd; // Object that handles the virtual K2 display.
    Mode state;
    Serial sport; // Object handling serial port interaction.
    byte[] response; // K2 command responses from the serial link.
    JLabel label; // This whole app is just a big ol' JLabel.
    static String k2pic = "images/k2panel.png"; // K2 pic.
    private boolean initialized = false;
    private boolean isPAon = false;

    /**
     * Create a K2 object and set up the serial port and virtual display
     * objects.
     */
    K2() {
        sport = new Serial(); // Serial port object.
        response = new byte[64]; // K2 response strings.
        dsd = new DSdecoder(); // Decodes K2 display command, DS.
        state = Mode.NORMAL;
    }

    public static void main(String[] args) {

        // Schedule a job for the event-dispatching thread:
        // creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                createAndShowGUI();
            }
        });
    }

    /**
     * Make the app's components and display them.
     */
    private static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("K2");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Set up the content pane.
        K2 controller = new K2();
        java.net.URL imgURL = K2.class.getResource(k2pic);
        controller.buildUI(frame.getContentPane(), new ImageIcon(imgURL, ""));

        //Display the window.
        frame.pack();
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        // Center panel on screen.
        frame.setLocation(((int) dim.getWidth() - frame.getWidth()) / 2,
                ((int) dim.getHeight() - frame.getHeight()) / 2);
        frame.setVisible(true);
    }

    /**
     * Make the front panel.
     */
    private void buildUI(Container container, ImageIcon image) {
        container.setLayout(new BoxLayout(container, BoxLayout.PAGE_AXIS));

        FrontPanelArea area = new FrontPanelArea(image, this);
        container.add(area);
    }

    /**
     * Set up the K2 front panel, add mouse listener, start timer. Basically,
     * make the K2 buttons and knobs and start listening to the user and the
     * real K2.
     */
    private class FrontPanelArea extends JLabel {

        JPopupMenu spMenu;
        private boolean startUp = true;
        private boolean directEntryMode = false;
        private int directEntryCount = -1;
        K2 controller;
        // Knob state.
        private int prevKeyerSpeed = 0;
        private double prevPower = 0;
        // Bargraph for S-units and ALC.
        private final Color ledColor = new Color(0x4d, 0xff, 0x4d);
        private final int bg = 0;
        Button button[], pushedButton, drawPhantom;
        KnobVFO vfoKnob;
        KnobKeyer keyerKnob;
        KnobOffset offsetKnob;
        KnobPower powerKnob;
        int timerDivider;
        // Timer used to distinguish between button taps and holds, as well
        // as retrieve display updates over the serial link from the real K2.
        private final Timer holdTimer;

        public FrontPanelArea(ImageIcon image, K2 controller) {
            super(image); //This component displays an image.
            this.controller = controller;
            setOpaque(true);
            setMinimumSize(new Dimension(10, 10)); // Don't hog space.
            this.setFocusable(true); // To get keyboard focus.

            MyListener myListener = new MyListener();
            addMouseListener(myListener);
            addMouseMotionListener(myListener);
            addKeyListener(myListener);

            button = mkButtons();
            pushedButton = null;
            drawPhantom = null;

            // Make knobs, setting center point & radius.
            vfoKnob = new KnobVFO(428, 207, 79);
            vfoKnob.setSteps(100);
            keyerKnob = new KnobKeyer(204, 169, 23);
            keyerKnob.setMin(9); // WPM
            keyerKnob.setMinStep(7);
            keyerKnob.setMax(50); // WPM
            keyerKnob.setMaxStep(55);
            keyerKnob.setSteps(61); // Settings per revolution.
            powerKnob = new KnobPower(288, 169, 23);
            offsetKnob = new KnobOffset(646, 254, 23);
            offsetKnob.setSteps(20); // Settings per revolution.

            repaint();

            // Get names of serial ports on system.
            String sp[] = sport.getSerialPortNames();
            spMenu = createPopupMenu(sp);

            timerDivider = 0; // Used to count (divide) timer intervals.
            holdTimer = new Timer(TIMEOUT, myListener.holdPerformer);
            holdTimer.setRepeats(true);
            holdTimer.start();
        }

        /**
         * Listen for button holds (not taps) amd knob turns.
         */
        private class MyListener extends MouseInputAdapter
                implements MouseMotionListener, KeyListener {

            public ActionListener holdPerformer = new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    holdTimer.stop();
                    timerDivider++; // Another timer interval counted.
                    if (timerDivider >= HOLD_DELAY) {
                        if (pushedButton != null) { // Button's been held.
                            if (pushedButton.getHoldFn().isEmpty()) {
                                System.exit(0); // On/off button hold exits.
                            }
                            sport.serialWrite(pushedButton.getHoldFn());
                            pushedButton = null;
                        }
                        timerDivider = 0;
                    } else if (vfoKnob.inUse()) {
                        tuneVFO();
                    } else if (keyerKnob.inUse()) {
                        setKeyerSpeed();
                    } else if (powerKnob.inUse()) {
                        setPower();
                    } else if (offsetKnob.inUse()) {
                        setOffset();
                    }
                    // Update the display, retrieve S-units.
                    if (inReceiveMode()) {
//                        sport.serialWrite("BG;", 40);
//                        sport.serialRead(response);
//                        if (response[0] == 'B' && response[1] == 'G') {
//                            String bgStr = new String(response, 2, 2);
//                            try {
//                                // Ignore garbled numbers with the 'catch'.
//                                bg = Integer.parseInt(bgStr);
//                            } catch (NumberFormatException ne) {
//                            }
//                        }
                        sport.serialWrite("DS;", 80);
                        sport.serialRead(response);
                    }
                    holdTimer.start();
                    repaint();
                }
            };

            /**
             * See if the K2 is in receive mode. We don't want to bother it if
             * it's in transmit.
             */
            private boolean inReceiveMode() {
                sport.serialWrite("TQ;", 30);
                sport.serialRead(response);
                return response[0] == 'T'
                        && response[1] == 'Q'
                        && response[2] == '0';
            }

            /**
             * See if user has requested a pop up menu to choose serial port, if
             * he's turning one of the knobs, or or if he's pressed a button.
             */
            @Override
            public void mousePressed(MouseEvent e) {

                int x = e.getX();
                int y = e.getY();

                // I can't figure out MS Windows pop up trigger, & so
                // explicitly look for button 3.
                if (e.isPopupTrigger() || e.getButton() == MouseEvent.BUTTON3) {
                    spMenu.show(e.getComponent(), x, y);
                    return;
                }

                if (vfoKnob.contains(x, y)) {
                    // Be sure K2 is in extended mode.
                    // We do it here in case the user started this program
                    // and only afterwards turned on his K2.
                    vfoKnob.setInUse(true);
                    vfoKnob.setCurrentSetting(x, y);
                } else if (keyerKnob.contains(x, y)) {
                    keyerKnob.setInUse(true);
                    keyerKnob.setCurrentSetting(x, y);
                } else if (powerKnob.contains(x, y)) {
                    powerKnob.setInUse(true);
                    powerKnob.setCurrentSetting(x, y);
                } else if (offsetKnob.contains(x, y)) {
                    if (e.getClickCount() >= 2) {
                        clearOffset();
                    } else {
                        offsetKnob.setInUse(true);
                        offsetKnob.setCurrentSetting(x, y);
                    }
                } else {
                    for (k2.Button b : button) {
                        if (b.contains(x, y)) {
                            pushedButton = b;
                        }
                    }

                }

                // After 0.5 s mouse press use the button's secondary fn.
                timerDivider = 0;
                holdTimer.restart();

                repaint();
            }

            @Override
            public void mouseClicked(MouseEvent e) {
                requestFocusInWindow();
                if (!initialized) {
                    holdTimer.stop();
                    initializeState();
                    holdTimer.start();
                }
            }

            /**
             * Update display if mouse just moved over a currently invisible
             * phantom button, and prepare to make it visible.
             */
            @Override
            public void mouseMoved(MouseEvent e) {

                int x = e.getX();
                int y = e.getY();

                drawPhantom = null;
                for (k2.Button b : button) {
                    if (b.isPhantomButton() && b.contains(x, y)) {
                        drawPhantom = b;
                    }
                }

                if (drawPhantom != null) {
                    repaint();
                }
            }

            /**
             * Mouse drags are only interesting if a knob is being turned. In
             * that case, update the appropriate knob's state.
             */
            @Override
            public void mouseDragged(MouseEvent e) {

                int x = e.getX();
                int y = e.getY();

                if (vfoKnob.inUse()) {
                    vfoKnob.setCurrentSetting(x, y);
                    vfoKnob.countStepsMoved();

                } else if (offsetKnob.inUse()) {
                    offsetKnob.setCurrentSetting(x, y);
                    offsetKnob.countStepsMoved();

                } else if (keyerKnob.inUse()) {
                    keyerKnob.setCurrentSetting(x, y);
                    keyerKnob.countStepsMoved();

                } else if (powerKnob.inUse()) {
                    powerKnob.setCurrentSetting(x, y);
                    powerKnob.countStepsMoved();
                }
            }

            /**
             * Handle release of knobs and button taps. Button holds are handled
             * when the timer goes off and sends control to the timer event
             * handler where pushedButton is set to null. So if we're here and
             * pushedButton is not null, then we definitely have a button tap,
             * not a hold.
             */
            @Override
            public void mouseReleased(MouseEvent e) {

                int x = e.getX();
                int y = e.getY();

                if (pushedButton != null) {
                    sport.serialWrite(pushedButton.getTapFn());
                    pushedButton = null;
                } else if (vfoKnob.inUse()) {
                    tuneVFO();
                    vfoKnob.setInUse(false);
                } else if (keyerKnob.inUse()) {
                    setKeyerSpeed();
                    keyerKnob.setInUse(false);
                } else if (powerKnob.inUse()) {
                    setPower();
                    powerKnob.setInUse(false);
                } else if (offsetKnob.inUse()) {
                    setOffset();
                    offsetKnob.setInUse(false);
                }
            }

            @Override
            public void keyTyped(KeyEvent e) {
            }

            /**
             * Handle the key pressed event from the text field. This makes the
             * program much nicer to use. With these commands everything can be
             * done from the keyboard without the need to ever touch the mouse.
             */
            @Override
            public void keyPressed(KeyEvent e) {
                if (!initialized) {
                    holdTimer.stop();
                    initializeState();
                    holdTimer.start();
                }
                int c = e.getKeyCode();
                switch (c) {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        int bIndex = 6 + c - '0'; // Button index for '0',...,'9'.
                        doDirectEntry((char) c); // Digit count book keeping.
                        depressButton(e, bIndex);
                        break;
                    case 'Q':
                        depressButton(e, 0);
                        break;
                    case 'W':
                        updateDisplayState();
                        depressButton(e, 1);
                        break;
                    case 'A':
                        depressButton(e, 2);
                        break;
                    case 'S':
                        depressButton(e, 3);
                        break;
                    case 'Z':
                        updateEditState(e.isShiftDown());
                        depressButton(e, 4);
                        break;
                    case 'X':
                        depressButton(e, 5);
                        break;
                    case 'F':
                        String disp = directEntryMode ? null : "-----@@@";
                        dsd.forceDisplay(disp, -1, -1);
                        directEntryCount = -1;
                        directEntryMode = !directEntryMode;
                        sport.serialWrite(button[17].getTapFn());
                        break;
                    case 'K':
                        int dir = e.isShiftDown() ? 1 : -1;
                        keyerKnob.setTurningDir(dir);
                        keyerKnob.setStepsMoved(1);
                        setKeyerSpeed();
                        break;
                    case 'P':
                        dir = e.isShiftDown() ? 1 : -1;
                        powerKnob.setTurningDir(dir);
                        powerKnob.setStepsMoved(1);
                        setPower();
                        break;
                    case 'R':
                        dir = e.isShiftDown() ? 1 : -1;
                        offsetKnob.setTurningDir(dir);
                        offsetKnob.setStepsMoved(1);
                        setOffset();
                        break;
                    case 'C':
                        clearOffset();
                        break;
                    case 'O':
                        if (e.isShiftDown()) {
                            System.exit(0);
                        }
                        break;
                    case KeyEvent.VK_UP:
                    case KeyEvent.VK_RIGHT:
                        int step = e.isShiftDown() ? 5 : 1;
                        vfoKnob.setTurningDir(1);
                        vfoKnob.setStepsMoved(step);
                        tuneVFO();
                        break;
                    case KeyEvent.VK_DOWN:
                    case KeyEvent.VK_LEFT:
                        step = e.isShiftDown() ? 5 : 1;
                        vfoKnob.setTurningDir(-1);
                        vfoKnob.setStepsMoved(step);
                        tuneVFO();
                        break;
                    default:
                        break;
                }
            }

            /**
             * Send the primary button command (tap) to the K2 on an unshifted
             * keyboard stroke, or send the secondary command (press and hold)
             * if the character is shifted.
             */
            private void depressButton(KeyEvent e, int b) {
                String cmd;
                if (e.isShiftDown()) {
                    cmd = button[b].getHoldFn();
                } else {
                    cmd = button[b].getTapFn();
                }
                sport.serialWrite(cmd);
            }

            /**
             * K2 direct frequency entry starts with five hyphens displayed on
             * the screen. As each digit is entered it becomes the rightmost
             * character and scrolls to the left for each new digit entered. A
             * frequency starting with a zero (160m) or 1 will have 4 more
             * digits, or a total of five. The others will have 3 more, or a
             * total of four. This method updates the count of still-to-come
             * digits with the directEntryCount variable. It also deals with
             * scrolling the display and clearing it when the frequency has been
             * fully entered.
             */
            private void doDirectEntry(char c) {
                if (directEntryMode) {
                    if (directEntryCount == -1) {
                        if (c == '0' || c == '1') {
                            directEntryCount = 4;
                        } else {
                            directEntryCount = 3;
                        }
                    } else {
                        directEntryCount--;
                    }
                    dsd.forceDisplayScroll(4, c);
                }
                if (directEntryCount == 0) {
                    directEntryCount = -1;
                    directEntryMode = false;
                    dsd.forceDisplay(null, -1, -1);
                }
            }

            /**
             * Handle the key released event from the text field.
             */
            @Override
            public void keyReleased(KeyEvent e) {
            }

            /**
             * Take care of tuning by the approriate amount depending not only
             * on how much the tuning knob was spun but also on whether the K2
             * is currently tuning to 10s, 100s, or 1000s of Hz.
             */
            private void tuneVFO() {

                if (vfoKnob.getStepsMoved() == 0) {
                    return; // Knob hasn't moved enough.
                }
                // Generate tuning command.
                String cmd = vfoKnob.getK2Command(
                        dsd.areTensUsed(),
                        dsd.areHundredsUsed());

                // Send commands to rig.
                for (int i = 0; i < vfoKnob.getStepsMoved(); i++) {
                    sport.serialWrite(cmd);
                }

                vfoKnob.clearSteps();
            }

            /**
             * Send appropriate command to set the keyer speed relative current
             * speed setting.
             */
            private void setKeyerSpeed() {

                int moved = keyerKnob.getStepsMoved();
                if (moved == 0) {
                    return; // Knob hasn't moved yet.
                }
                int wpm = (int) keyerKnob.getSetting();
                wpm += keyerKnob.getTurningDir() == 1 ? moved : -moved;
                if (wpm == prevKeyerSpeed) {
                    return; // Don't resend identical command.
                }
                keyerKnob.setSetting(wpm);
                keyerKnob.calcSetPoint();
                prevKeyerSpeed = (int) keyerKnob.getSetting();
                sport.serialWrite(keyerKnob.getK2Command());
                keyerKnob.clearSteps();
            }

            /**
             * Send appropriate command to set the power relative current power
             * setting.
             */
            private void setPower() {

                int moved = powerKnob.getStepsMoved();
                if (moved == 0) {
                    return; // Knob hasn't moved yet.
                }
                double pwr = powerKnob.getSetting();
                double delta;
                if (isPAon && pwr <= 10) {
                    delta = moved * 0.5;
                } else if (!isPAon) {
                    delta = moved * 0.1;
                } else {
                    delta = moved;
                }
                pwr += powerKnob.getTurningDir() == 1 ? delta : -delta;
                if (pwr == prevPower) {
                    return; // Don't resend identical command.
                }
                powerKnob.setSetting(pwr);
                powerKnob.calcSetPoint();
                prevPower = powerKnob.getSetting();
                powerKnob.setPAState(isPAon);
                sport.serialWrite(powerKnob.getK2Command());
                powerKnob.clearSteps();
            }

            /**
             * Send appropriate command to spin the offset knob up or down.
             */
            private void clearOffset() {
                sport.serialWrite(offsetKnob.getClearCommand());
            }

            /**
             * Send appropriate command to spin the offset knob up or down.
             */
            private void setOffset() {

                if (offsetKnob.getStepsMoved() == 0) {
                    return; // Knob hasn't moved enough.
                }
                // Generate offset command.
                String cmd = offsetKnob.getK2Command();

                // Send commands to rig.
                for (int i = 0; i < offsetKnob.getStepsMoved(); i++) {
                    sport.serialWrite(cmd);
                }

                offsetKnob.clearSteps();
            }
        }

        /**
         * Draw it!
         */
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g); //paints the background and image

            if (startUp) {
                // There must be a better way to do this.  I want to show
                // first "DE AB3AP" for 1 second, then go to normal operation.
                // Shameless self promotion.
                dsd.forceDisplay("DE AB3AP", -1, -1);
                dsd.forceDisplay(null, -1, -1);
                startUp = false;
            } else // Mimic K2 display.
            {
                dsd.display(g, this, response, state);
            }
            // Draw S-units/ALC.
            drawBarGraph(g, bg);
            // Draw lines on dials.
            if (initialized) {
                Graphics2D g2 = (Graphics2D) g;
                Stroke os = g2.getStroke();
                g2.setStroke(new BasicStroke(2f));
                g.setColor(Color.white);
                // Draw keyer setting.
                Point setPt = keyerKnob.getSetPoint();
                g.drawLine(keyerKnob.getX(), keyerKnob.getY(),
                        (int) setPt.getX(), (int) setPt.getY());
                // Draw power setting.
                setPt = powerKnob.getSetPoint();
                g.drawLine(powerKnob.getX(), powerKnob.getY(),
                        (int) setPt.getX(), (int) setPt.getY());
                g2.setStroke(os);
            }
            // Draw any phantom buttons that are active.
            if (drawPhantom != null) {
                g.drawImage(drawPhantom.getPhantomImage(),
                        drawPhantom.getPhantomX(), drawPhantom.getPhantomY(),
                        this);
            }

        }

        /**
         * Make all the buttons by specifying geometry and primary and secondary
         * functionality. Functionality strings are the actual K2 commands
         * corresponding to the buttons. Phantom button images and locations are
         * also given where needed.
         */
        public Button[] mkButtons() {

            Button[] b = new Button[24];
            b[0] = new Button("SW01;", "SW17;", 31, 28, 49, 25); // BAND+
            b[1] = new Button("SW02;", "SW18;", 94, 28, 49, 25); // DISPLAY
            b[2] = new Button("SW03;", "SW19;", 31, 94, 49, 25); // BAND-
            b[3] = new Button("SW04;", "SW20;", 94, 94, 49, 25); // ANT 1/2
            b[4] = new Button("SW05;", "SW21;", 31, 160, 49, 25); // MENU
            b[5] = new Button("SW06;", "SW22;", 94, 160, 49, 25); // NB
            b[6] = new Button("SW07;", "SW23;", 531, 234, 34, 35); // RATE
            b[7] = new Button("SW08;", "SW24;", 592, 28, 49, 25); // MODE
            b[8] = new Button("SW09;", "SW25;", 650, 28, 49, 25); // A/B
            b[9] = new Button("SW10;", "SW26;", 711, 28, 49, 25); // A=B
            b[10] = new Button("SW11;", "SW27;", 592, 94, 49, 25); // PR/AT
            b[11] = new Button("SW12;", "SW28;", 650, 94, 49, 25); // AGC
            b[12] = new Button("SW13;", "SW29;", 711, 94, 49, 25); // XFIL
            b[13] = new Button("SW14;", "SW30;", 592, 160, 49, 25); // RIT
            b[14] = new Button("SW15;", "SW31;", 650, 160, 49, 25); // XIT
            b[15] = new Button("SW16;", "SW32;", 711, 160, 49, 25); // MSG
            b[16] = new Button("", "", 711, 220, 34, 35); // ON
            b[17] = new Button("SW34;", "SW34;", 13, 63, 18, 21); // F
            b[18] = new Button("SW35;", "SW35;", 143, 64, 29, 20); // F/R
            b[18].setPhantomImage("images/fr.png", 140, 42);
            b[19] = new Button("SW33;", "SW33;", 656, 127, 37, 22); // RIT
            b[19].setPhantomImage("images/rit.png", 585, 119);
            b[20] = new Button("SW36;", "SW36;", 628, 72, 34, 20); // AGC
            b[20].setPhantomImage("images/agc.png", 588, 71);
            b[21] = new Button("SW37;", "SW37;", 690, 72, 36, 21); // FIL
            b[21].setPhantomImage("images/fil.png", 648, 71);
            b[22] = new Button("SW80;", "SW80;", 753, 64, 27, 21); // NF
            b[22].setPhantomImage("images/nf.png", 753, 37);
            b[23] = new Button("SW81;", "SW81;", 753, 130, 27, 22); // NR
            b[23].setPhantomImage("images/nr.png", 753, 103);

            return b;
        }

        /**
         * Move to the secondary menu entry for PA (power amp) to see if it's
         * turned on or not. Simply looking at a PC response doesn't give the
         * correct info. For 10W or less, PC says the PA is not on even if it is
         * enabled.
         */
        public boolean isPAEnabled() {
            sport.serialWrite("SW66;SW05;", 100); // PA, MENU.
            sport.serialWrite("DS;", 100); // Get display to see if PA is on.
            sport.serialWrite("SW05;"); // Exit MENU mode.
            for (int i = 0; i < response.length; i++) {
                response[i] = '\0';
            }
            sport.serialRead(response);
            return response[8] == 'N';
        }

        public double getPower() {
            isPAon = isPAEnabled();
            sport.serialWrite("PC;".getBytes(), 100);
            sport.serialRead(response);
            boolean isPAinUse = response[5] == '1';
            String pwrStr = new String(response, 2, 3);
            double pwr = Double.parseDouble(pwrStr);
            if (!isPAinUse) {
                pwr /= 10;
            }
            if (isPAon) {
                powerKnob.setMin(0);
                powerKnob.setMinStep(20);
                powerKnob.setMax(111); // 111 W
                powerKnob.setMaxStep(129);
                powerKnob.setSteps(150); // Settings per revolution.
            } else {
                powerKnob.setMin(0.1); // 0.1 W
                powerKnob.setMinStep(25);
                powerKnob.setMax(15.0); // 15.2 W
                powerKnob.setMaxStep(175);
                powerKnob.setSteps(200); // Settings per revolution.
            }
            return pwr;
        }

        public int getKeyerSpeed() {
            sport.serialWrite("KS;".getBytes(), 100);
            sport.serialRead(response);
            String spdStr = new String(response, 2, 3);
            int spd = Integer.parseInt(spdStr);
            return spd;
        }

        public void initializeState() {
            sport.serialWrite("K22;", 100); // Use extended command mode.
            sport.serialWrite("K2;", 100); // Verify mode.
            byte[] resp = new byte[32];
            sport.serialRead(resp);
            String r = new String(resp);
            if (r.indexOf("K22;") == -1) {
                return;
            }
            initialized = true;
            keyerKnob.setSetting(getKeyerSpeed());
            keyerKnob.calcSetPoint();
            powerKnob.setSetting(getPower());
            powerKnob.calcSetPoint();
        }

        private void updateEditState(boolean isShiftDown) {
            if (isShiftDown) {
                switch (state) {
                    case EDIT:
                        state = Mode.NORMAL;
                        break;
                    default:
                        state = Mode.EDIT;
                        break;
                }
            } else {
                switch (state) {
                    case NORMAL:
                    case EDIT:
                        state = Mode.MENU;
                        break;
                    default:
                        state = Mode.NORMAL;
                        break;
                }
            }
        }

        private void updateDisplayState() {
            switch (state) {
                case NORMAL:
                    state = Mode.DSP;
                    break;
                case DSP:
                    state = Mode.POWER;
                    break;
                case POWER:
                    state = Mode.NORMAL;
                    break;
                default:
                    // No change if in MENU or EDIT.
                    break;
            }
        }

        /**
         * Draw the S-units bar graph on the display. Just a bunch of
         * rectangles, nothing tricky.
         */
        private void drawBarGraph(Graphics g, int bg) {
            if (bg == 0 || bg == 12) {
                return;
            }
            g.setColor(ledColor);
            if (bg < 11) {
                // DOT mode.
                g.fillRect(180 + 10 * (bg - 1), 51, 6, 20);
            } else {
                // BAR mode.
                bg -= 12;
                for (int i = 0; i < bg; i++) {
                    g.fillRect(180 + 10 * i, 51, 6, 20);
                }
            }
        }

        /**
         * Respond to user's pop up menu choice by opening serial port he chose.
         */
        private void popupPressed(MouseEvent e) {
            String port = ((JMenuItem) e.getSource()).getText();
            sport.setSerialPort("K2", port);
        }

        /**
         * Make a pop up menu and populate it with the serial ports discovered
         * on this system.
         */
        public JPopupMenu createPopupMenu(String portNames[]) {
            JMenuItem menuItem;

            JPopupMenu popup = new JPopupMenu();
            JLabel name = new JLabel("Choose K2 serial port");
            popup.add(name);
            popup.addSeparator();
            for (String portName : portNames) {
                menuItem = new JMenuItem(portName);
                menuItem.addMouseListener(new MouseAdapter() {

                    @Override
                    public void mousePressed(MouseEvent e) {
                        popupPressed(e);
                    }
                });
                popup.add(menuItem);
            }

            return popup;
        }
    }
}
