package digital;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

/**
 * The W1State class maintains the operational state of the W1 meter. For
 * example, are LEDs on or off, what is the current scale range, and so on. It
 * is also used to display the state on the virtual W1 display.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br> March 2007
 */
public class W1State {

    private Color meterColor;
    private Font font = new Font("SansSerif", Font.PLAIN, 9);
    private String range, drop, pwr, peak, decay, led;
    private double max;
    private int rangex, rangey, dropx, dropy, pwrx, pwry,
            peakx, peaky, decayx, decayy, ledx, ledy;

    /**
     * W1State is created with no initial default state. State must be set
     * explicitly with other methods of this object..
     */
    W1State() {
        range = drop = pwr = peak = decay = led = "";
    }

    /**
     * Get color used to paint state settings on the meter .
     */
    public Color getMeterColor() {
        return meterColor;
    }

    /**
     * Set color used to paint state settings on the meter .
     */
    public void setMeterColor(Color c) {
        meterColor = c;
    }

    /**
     * Return range label of meter. For instance, the W1 uses ranges of
     * 0.150-1.40W, 1.50-14.0W, and 15.0-140W named with the Strings LO, MED,
     * and HI. This label is displayed on the meter.
     */
    public String getRange() {
        return range;
    }

    /**
     * Set the range label of this meter.
     */
    public void setRange(String s) {
        range = new String(s);
    }

    /**
     * Set the range label of this meter.
     */
    public void setRange(char r) {
        switch (r) {
            case '0':
                max = 0.;
                range = "AUTO";
                break;
            case '1':
                max = 1.4;
                range = "LO";
                break;
            case '2':
                range = "MED";
                max = 14.;
                break;
            case '3':
                range = "HI";
                max = 140.;
                break;
            default:
                break;
        }
    }

    /**
     * Indicate where, in pixels, the range label is to be written.
     */
    public void setRangeLocation(int x, int y) {
        rangex = x;
        rangey = y;
    }

    public double getRangeMax() {
        return max;
    }

    public double getNextRangeMax(double reading) {
        double max;

        if (reading <= 1.4) {
            max = 1.4;
        } else if (reading <= 14.) {
            max = 14.;
        } else {
            max = 140.;
        }
        return max;
    }

    /**
     * Get the label indicating the current range drop rate setting.
     */
    public String getDrop() {
        return drop;
    }

    /**
     * Set the label indicating the current range drop rate setting.
     */
    public void setDrop(String s) {
        drop = new String(s);
    }

    /**
     * Set the label indicating the current range drop rate setting. This method
     * takes as input the unique character returned from the W1 indicating
     * current drop rate.
     */
    public void setDrop(char d) {
        switch (d) {
            case 'S':
                drop = "SLO";
                break;
            case 'M':
                drop = "MED";
                break;
            case 'F':
                drop = "FAST";
                break;
            default:
                drop = "";
                break;
        }
    }

    /**
     * Indicate where, in pixels, the range drop label is to be written.
     */
    public void setDropLocation(int x, int y) {
        dropx = x;
        dropy = y;
    }

    /**
     * Get label indicating current power measurement, AVG or PEP.
     */
    public String getPwr() {
        return pwr;
    }

    /**
     * Set label indicating current power measurement, AVG or PEP.
     */
    public void setPwr(String s) {
        pwr = new String(s);
    }

    /**
     * Set label indicating current power measurement, AVG or PEP. The parameter
     * 'p' is expected to be the second character in the response to a W1 "M" or
     * "N" command. The response is NA or NP for average or PEP.
     */
    public void setPwr(char p) {
        switch (p) {
            case 'A':
                pwr = "AVG";
                break;
            case 'P':
                pwr = "PEP";
                break;
            default:
                pwr = "";
                break;
        }
    }

    /**
     * Indicate where, in pixels, the power label is to be written.
     */
    public void setPwrLocation(int x, int y) {
        pwrx = x;
        pwry = y;
    }

    /**
     * Get label indicating if regular or peak-hold display is in effect.
     */
    public String getPeak() {
        return peak;
    }

    /**
     * Set label indicating if regular or peak-hold display is in effect.
     */
    public void setPeak(String s) {
        peak = new String(s);
    }

    /**
     * Set label indicating if regular or peak-hold display is in effect.
     */
    public void setPeak(char p) {
        switch (p) {
            case 'R':
                peak = "";
                break;
            case 'H':
                peak = "HOLD";
                break;
            default:
                break;
        }
    }

    /**
     * Indicate where, in pixels, the peak-hold label is to be written.
     */
    public void setPeakLocation(int x, int y) {
        peakx = x;
        peaky = y;
    }

    /**
     * Get label indicating LED decay speed.
     */
    public String getDecay() {
        return decay;
    }

    /**
     * Set label indicating LED decay speed.
     */
    public void setDecay(String s) {
        decay = new String(s);
    }

    /**
     * Set label indicating LED decay speed.
     */
    public void setDecay(char d) {
        switch (d) {
            case 'S':
                decay = "SLO";
                break;
            case 'M':
                decay = "MED";
                break;
            case 'F':
                decay = "FAST";
                break;
            default:
                decay = "";
                break;
        }
    }

    /**
     * Indicate where, in pixels, the led decay rate label is to be written.
     */
    public void setDecayLocation(int x, int y) {
        decayx = x;
        decayy = y;
    }

    /**
     * Get label indicating whether LEDs are on or off.
     */
    public String getLED() {
        return led;
    }

    /**
     * Set label indicating whether LEDs are on or off.
     */
    public void setLED(String s) {
        led = new String(s);
    }

    /**
     * Indicate where, in pixels, the led on/off label is to be written.
     */
    public void setLEDLocation(int x, int y) {
        ledx = x;
        ledy = y;
    }

    /**
     * Update the on-screen state of the W1 by using the state maintained in the
     * corresponding button. For instance, some buttons cycle through values
     * when repeatedly pushed, and it is only the button that knows the current
     * setting. Maybe that state info should be moved into this object...
     */
    public void setLabelPrimary(String buttonName, String displayLabel) {
        if (buttonName.equals("range/drop")) {
            setRange(displayLabel);
        } else if (buttonName.equals("pwr/peak")) {
            setPwr(displayLabel);
        } else if (buttonName.equals("decay/led")) {
            setDecay(displayLabel);
        }
    }

    /**
     * Update the on-screen state of the W1 by using the state maintained in the
     * corresponding button for its secondary function.
     */
    public void setLabelSecondary(String buttonName, String displayLabel) {
        if (buttonName.equals("range/drop")) {
            setDrop(displayLabel);
        } else if (buttonName.equals("pwr/peak")) {
            setPeak(displayLabel);
        } else if (buttonName.equals("decay/led")) {
            setLED(displayLabel);
        }
    }

    /**
     * Draw the state labels on the W1.
     */
    public void display(Graphics g) {
        g.setFont(font);
        g.setColor(meterColor);
        g.drawString(range, rangex, rangey);
        g.drawString(drop, dropx, dropy);
        g.drawString(pwr, pwrx, pwry);
        g.drawString(peak, peakx, peaky);
        g.drawString(decay, decayx, decayy);
        g.drawString(led, ledx, ledy);
    }
}
